"""Store client for supervisor."""

from .client import _SupervisorComponentClient
from .const import ResponseType
from .models.addons import (
    Repository,
    StoreAddon,
    StoreAddonComplete,
    StoreAddonsList,
    StoreAddonUpdate,
    StoreAddRepository,
    StoreInfo,
)


class StoreClient(_SupervisorComponentClient):
    """Handles store access in Supervisor."""

    async def info(self) -> StoreInfo:
        """Get store info."""
        result = await self._client.get("store")
        return StoreInfo.from_dict(result.data)

    async def addons_list(self) -> list[StoreAddon]:
        """Get list of store addons."""
        result = await self._client.get("store/addons")
        return StoreAddonsList.from_dict(result.data).addons

    async def addon_info(self, addon: str) -> StoreAddonComplete:
        """Get store addon info."""
        result = await self._client.get(f"store/addons/{addon}")
        return StoreAddonComplete.from_dict(result.data)

    async def addon_changelog(self, addon: str) -> str:
        """Get addon changelog."""
        result = await self._client.get(
            f"store/addons/{addon}/changelog", response_type=ResponseType.TEXT
        )
        return result.data

    async def addon_documentation(self, addon: str) -> str:
        """Get addon documentation."""
        result = await self._client.get(
            f"store/addons/{addon}/documentation", response_type=ResponseType.TEXT
        )
        return result.data

    async def install_addon(self, addon: str) -> None:
        """Install an addon."""
        await self._client.post(f"store/addons/{addon}/install", timeout=None)

    async def update_addon(
        self, addon: str, options: StoreAddonUpdate | None = None
    ) -> None:
        """Update an addon to latest version."""
        await self._client.post(
            f"store/addons/{addon}/update",
            json=options.to_dict() if options else None,
            timeout=None,
        )

    async def reload(self) -> None:
        """Reload the store."""
        await self._client.post("store/reload")

    async def repositories_list(self) -> list[Repository]:
        """Get list of repositories."""
        result = await self._client.get("store/repositories")
        # This API is inconsistent with Supervisor's API model, data should be
        # a dictionary with a "repositories" field. It would break the CLI like
        # this but the CLI doesn't use it so it went unnoticed.
        return [Repository.from_dict(repo) for repo in result.data]

    async def repository_info(self, repository: str) -> Repository:
        """Get repository info."""
        result = await self._client.get(f"store/repositories/{repository}")
        return Repository.from_dict(result.data)

    async def add_repository(self, options: StoreAddRepository) -> None:
        """Add a repository to the store."""
        await self._client.post("store/repositories", json=options.to_dict())

    async def remove_repository(self, repository: str) -> None:
        """Remove a repository from the store."""
        await self._client.delete(f"store/repositories/{repository}")

    # Omitted for now - Icon/Logo endpoints



"""Models for Home Assistant."""

from dataclasses import dataclass
from ipaddress import IPv4Address

from .base import DEFAULT, ContainerStats, Options, Request, ResponseData

# --- OBJECTS ----


@dataclass(frozen=True, slots=True)
class HomeAssistantInfo(ResponseData):
    """HomeAssistantInfo model."""

    version: str | None
    version_latest: str | None
    update_available: bool
    machine: str
    ip_address: IPv4Address
    arch: str
    image: str
    boot: bool
    port: int
    ssl: bool
    watchdog: bool
    audio_input: str
    audio_output: str
    backups_exclude_database: bool


@dataclass(frozen=True, slots=True)
class HomeAssistantStats(ContainerStats):
    """HomeAssistantStats model."""


@dataclass(frozen=True, slots=True)
class HomeAssistantOptions(Options):
    """HomeAssistantOptions model."""

    boot: bool | None = None
    image: str | None = DEFAULT  # type: ignore[assignment]
    port: int | None = None
    ssl: bool | None = None
    watchdog: bool | None = None
    refresh_token: str | None = DEFAULT  # type: ignore[assignment]
    audio_input: str | None = DEFAULT  # type: ignore[assignment]
    audio_output: str | None = DEFAULT  # type: ignore[assignment]
    backups_exclude_database: bool | None = None


@dataclass(frozen=True, slots=True)
class HomeAssistantUpdateOptions(Options):
    """HomeAssistantUpdateOptions model."""

    version: str | None = None
    backup: bool | None = None


@dataclass(frozen=True, slots=True)
class HomeAssistantRestartOptions(Options):
    """HomeAssistantRestartOptions model."""

    safe_mode: bool | None = None
    force: bool | None = None


@dataclass(frozen=True, slots=True)
class HomeAssistantRebuildOptions(Options):
    """HomeAssistantRebuildOptions model."""

    safe_mode: bool | None = None
    force: bool | None = None


@dataclass(frozen=True, slots=True)
class HomeAssistantStopOptions(Request):
    """HomeAssistantStopOptions model."""

    force: bool



"""Models for host APIs."""

from dataclasses import dataclass
from datetime import datetime
from enum import StrEnum

from .base import Request, ResponseData
from .root import HostFeature

# --- ENUMS ----


class ServiceState(StrEnum):
    """ServiceState type.

    The service state is determined by systemd, not supervisor. The list below
    is pulled from `systemctl --state=help`. It may be incomplete and it may
    change based on the host. Therefore within a list of services there may be
    some with a state not in this list parsed as string. If you find this
    please create an issue or pr to get the state added.
    """

    ACTIVE = "active"
    RELOADING = "reloading"
    INACTIVE = "inactive"
    FAILED = "failed"
    ACTIVATING = "activating"
    DEACTIVATING = "deactivating"
    MAINTENANCE = "maintenance"


# --- OBJECTS ----


@dataclass(frozen=True, slots=True)
class HostInfo(ResponseData):
    """HostInfo model."""

    agent_version: str | None
    apparmor_version: str | None
    chassis: str | None
    virtualization: str | None
    cpe: str | None
    deployment: str | None
    disk_free: float
    disk_total: float
    disk_used: float
    disk_life_time: float
    features: list[HostFeature]
    hostname: str | None
    llmnr_hostname: str | None
    kernel: str | None
    operating_system: str | None
    timezone: str | None
    dt_utc: datetime | None
    dt_synchronized: bool | None
    use_ntp: bool | None
    startup_time: float | None
    boot_timestamp: int | None
    broadcast_llmnr: bool | None
    broadcast_mdns: bool | None


@dataclass(frozen=True, slots=True)
class ShutdownOptions(Request):
    """ShutdownOptions model."""

    force: bool


@dataclass(frozen=True, slots=True)
class RebootOptions(Request):
    """RebootOptions model."""

    force: bool


@dataclass(frozen=True, slots=True)
class HostOptions(Request):
    """HostOptions model."""

    hostname: str


@dataclass(frozen=True, slots=True)
class Service(ResponseData):
    """Service model."""

    name: str
    description: str
    state: ServiceState | str


@dataclass(frozen=True, slots=True)
class ServiceList(ResponseData):
    """ServiceList model."""

    services: list[Service]



"""Discovery client for supervisor."""

from uuid import UUID

from .client import _SupervisorComponentClient
from .const import TIMEOUT_60_SECONDS, ResponseType
from .models.discovery import Discovery, DiscoveryConfig, DiscoveryList, SetDiscovery


class DiscoveryClient(_SupervisorComponentClient):
    """Handles discovery access in supervisor."""

    async def list(self) -> list[Discovery]:
        """List discovered active services."""
        result = await self._client.get("discovery", timeout=TIMEOUT_60_SECONDS)
        return DiscoveryList.from_dict(result.data).discovery

    async def get(self, uuid: UUID) -> Discovery:
        """Get discovery details for a service."""
        result = await self._client.get(f"discovery/{uuid.hex}")
        return Discovery.from_dict(result.data)

    async def delete(self, uuid: UUID) -> None:
        """Remove discovery for a service."""
        await self._client.delete(f"discovery/{uuid.hex}")

    async def set(self, config: DiscoveryConfig) -> UUID:
        """Inform supervisor of an available service."""
        result = await self._client.post(
            "discovery", json=config.to_dict(), response_type=ResponseType.JSON
        )
        return SetDiscovery.from_dict(result.data).uuid



"""Tests for client."""

import pytest

from aiohasupervisor.client import _SupervisorClient
from aiohasupervisor.exceptions import SupervisorError

from .const import SUPERVISOR_URL


@pytest.mark.parametrize("method", ["get", "post", "put", "delete"])
async def test_path_manipulation_blocked(method: str) -> None:
    """Test path manipulation prevented."""
    client = _SupervisorClient(SUPERVISOR_URL, "abc123", 10)
    action = getattr(client, method)
    with pytest.raises(SupervisorError):
        # absolute path
        await action("/test/../bad")
    with pytest.raises(SupervisorError):
        # relative path
        await action("test/../bad")
    with pytest.raises(SupervisorError):
        # relative path with percent encoding
        await action("test/%2E%2E/bad")



"""Resolution center client for supervisor."""

from uuid import UUID

from .client import _SupervisorComponentClient
from .models.resolution import (
    CheckOptions,
    CheckType,
    ResolutionInfo,
    Suggestion,
    SuggestionsList,
)


class ResolutionClient(_SupervisorComponentClient):
    """Handles resolution center access in supervisor."""

    async def info(self) -> ResolutionInfo:
        """Get resolution center info."""
        result = await self._client.get("resolution/info")
        return ResolutionInfo.from_dict(result.data)

    async def check_options(
        self, check: CheckType | str, options: CheckOptions
    ) -> None:
        """Set options for a check."""
        await self._client.post(
            f"resolution/check/{check}/options", json=options.to_dict()
        )

    async def run_check(self, check: CheckType | str) -> None:
        """Run a check."""
        await self._client.post(f"resolution/check/{check}/run")

    async def apply_suggestion(self, suggestion: UUID) -> None:
        """Apply a suggestion."""
        await self._client.post(f"resolution/suggestion/{suggestion.hex}", timeout=None)

    async def dismiss_suggestion(self, suggestion: UUID) -> None:
        """Dismiss a suggestion."""
        await self._client.delete(f"resolution/suggestion/{suggestion.hex}")

    async def dismiss_issue(self, issue: UUID) -> None:
        """Dismiss an issue."""
        await self._client.delete(f"resolution/issue/{issue.hex}")

    async def suggestions_for_issue(self, issue: UUID) -> list[Suggestion]:
        """Get suggestions for issue."""
        result = await self._client.get(f"resolution/issue/{issue.hex}/suggestions")
        return SuggestionsList.from_dict(result.data).suggestions

    async def healthcheck(self) -> None:
        """Run a healthcheck."""
        await self._client.post("resolution/healthcheck")



"""Models for supervisor client."""

from aiohasupervisor.models.addons import (
    AddonBoot,
    AddonBootConfig,
    AddonsConfigValidate,
    AddonsOptions,
    AddonsSecurityOptions,
    AddonsStats,
    AddonStage,
    AddonStartup,
    AddonState,
    AddonsUninstall,
    AppArmor,
    Capability,
    CpuArch,
    InstalledAddon,
    InstalledAddonComplete,
    Repository,
    StoreAddon,
    StoreAddonComplete,
    StoreAddonUpdate,
    StoreAddRepository,
    StoreInfo,
    SupervisorRole,
)
from aiohasupervisor.models.backups import (
    AddonSet,
    Backup,
    BackupAddon,
    BackupComplete,
    BackupContent,
    BackupJob,
    BackupsInfo,
    BackupsOptions,
    BackupType,
    DownloadBackupOptions,
    Folder,
    FreezeOptions,
    FullBackupOptions,
    FullRestoreOptions,
    NewBackup,
    PartialBackupOptions,
    PartialRestoreOptions,
    RemoveBackupOptions,
    UploadBackupOptions,
)
from aiohasupervisor.models.discovery import (
    Discovery,
    DiscoveryConfig,
)
from aiohasupervisor.models.homeassistant import (
    HomeAssistantInfo,
    HomeAssistantOptions,
    HomeAssistantRebuildOptions,
    HomeAssistantRestartOptions,
    HomeAssistantStats,
    HomeAssistantStopOptions,
    HomeAssistantUpdateOptions,
)
from aiohasupervisor.models.host import (
    HostInfo,
    HostOptions,
    RebootOptions,
    Service,
    ServiceState,
    ShutdownOptions,
)
from aiohasupervisor.models.mounts import (
    CIFSMountRequest,
    CIFSMountResponse,
    MountCifsVersion,
    MountsInfo,
    MountsOptions,
    MountState,
    MountType,
    MountUsage,
    NFSMountRequest,
    NFSMountResponse,
)
from aiohasupervisor.models.network import (
    AccessPoint,
    AuthMethod,
    DockerNetwork,
    InterfaceMethod,
    InterfaceType,
    IPv4,
    IPv4Config,
    IPv6,
    IPv6Config,
    NetworkInfo,
    NetworkInterface,
    NetworkInterfaceConfig,
    Vlan,
    VlanConfig,
    Wifi,
    WifiConfig,
    WifiMode,
)
from aiohasupervisor.models.os import (
    BootSlot,
    BootSlotName,
    DataDisk,
    GreenInfo,
    GreenOptions,
    MigrateDataOptions,
    OSInfo,
    OSUpdate,
    RaucState,
    SetBootSlotOptions,
    YellowInfo,
    YellowOptions,
)
from aiohasupervisor.models.resolution import (
    Check,
    CheckOptions,
    CheckType,
    ContextType,
    Issue,
    IssueType,
    ResolutionInfo,
    Suggestion,
    SuggestionType,
    UnhealthyReason,
    UnsupportedReason,
)
from aiohasupervisor.models.root import (
    AvailableUpdate,
    HostFeature,
    LogLevel,
    RootInfo,
    SupervisorState,
    UpdateChannel,
    UpdateType,
)
from aiohasupervisor.models.supervisor import (
    SupervisorInfo,
    SupervisorOptions,
    SupervisorStats,
    SupervisorUpdateOptions,
)

__all__ = [
    "HostFeature",
    "SupervisorState",
    "UpdateChannel",
    "LogLevel",
    "UpdateType",
    "RootInfo",
    "AvailableUpdate",
    "AddonStage",
    "AddonStartup",
    "AddonBoot",
    "AddonBootConfig",
    "CpuArch",
    "Capability",
    "AppArmor",
    "SupervisorRole",
    "AddonState",
    "StoreAddon",
    "StoreAddonComplete",
    "InstalledAddon",
    "InstalledAddonComplete",
    "AddonsOptions",
    "AddonsConfigValidate",
    "AddonsSecurityOptions",
    "AddonsStats",
    "AddonsUninstall",
    "Repository",
    "StoreInfo",
    "StoreAddonUpdate",
    "StoreAddRepository",
    "Check",
    "CheckOptions",
    "CheckType",
    "ContextType",
    "Issue",
    "IssueType",
    "ResolutionInfo",
    "Suggestion",
    "SuggestionType",
    "UnhealthyReason",
    "UnsupportedReason",
    "SupervisorInfo",
    "SupervisorOptions",
    "SupervisorStats",
    "SupervisorUpdateOptions",
    "HomeAssistantInfo",
    "HomeAssistantOptions",
    "HomeAssistantRebuildOptions",
    "HomeAssistantRestartOptions",
    "HomeAssistantStats",
    "HomeAssistantStopOptions",
    "HomeAssistantUpdateOptions",
    "RaucState",
    "BootSlotName",
    "BootSlot",
    "OSInfo",
    "OSUpdate",
    "MigrateDataOptions",
    "DataDisk",
    "SetBootSlotOptions",
    "GreenInfo",
    "GreenOptions",
    "YellowInfo",
    "YellowOptions",
    "AddonSet",
    "Backup",
    "BackupAddon",
    "BackupComplete",
    "BackupContent",
    "BackupJob",
    "BackupsInfo",
    "BackupsOptions",
    "BackupType",
    "DownloadBackupOptions",
    "Folder",
    "FreezeOptions",
    "FullBackupOptions",
    "FullRestoreOptions",
    "NewBackup",
    "PartialBackupOptions",
    "PartialRestoreOptions",
    "RemoveBackupOptions",
    "UploadBackupOptions",
    "Discovery",
    "DiscoveryConfig",
    "AccessPoint",
    "AuthMethod",
    "DockerNetwork",
    "InterfaceMethod",
    "InterfaceType",
    "IPv4",
    "IPv4Config",
    "IPv6",
    "IPv6Config",
    "NetworkInfo",
    "NetworkInterface",
    "NetworkInterfaceConfig",
    "Vlan",
    "VlanConfig",
    "Wifi",
    "WifiConfig",
    "WifiMode",
    "HostInfo",
    "HostOptions",
    "RebootOptions",
    "Service",
    "ServiceState",
    "ShutdownOptions",
    "CIFSMountRequest",
    "CIFSMountResponse",
    "MountCifsVersion",
    "MountsInfo",
    "MountsOptions",
    "MountState",
    "MountType",
    "MountUsage",
    "NFSMountRequest",
    "NFSMountResponse",
]



"""Models for Supervisor backups."""

from abc import ABC
from dataclasses import dataclass
from datetime import datetime
from enum import StrEnum

from .base import DEFAULT, Request, ResponseData

# --- ENUMS ----


class BackupType(StrEnum):
    """BackupType type."""

    FULL = "full"
    PARTIAL = "partial"


class Folder(StrEnum):
    """Folder type."""

    SHARE = "share"
    ADDONS = "addons/local"
    SSL = "ssl"
    MEDIA = "media"


class AddonSet(StrEnum):
    """AddonSet type."""

    ALL = "ALL"


# --- OBJECTS ----


@dataclass(frozen=True, slots=True)
class BackupContent(ResponseData):
    """BackupContent model."""

    homeassistant: bool
    addons: list[str]
    folders: list[Folder]


@dataclass(frozen=True)
class BackupBaseFields(ABC):
    """BackupBaseFields ABC type."""

    slug: str
    name: str
    date: datetime
    type: BackupType
    size: float
    size_bytes: int
    location: str | None
    locations: set[str | None]
    protected: bool
    compressed: bool


@dataclass(frozen=True, slots=True)
class Backup(BackupBaseFields, ResponseData):
    """Backup model."""

    content: BackupContent


@dataclass(frozen=True, slots=True)
class BackupAddon(ResponseData):
    """BackupAddon model."""

    slug: str
    name: str
    version: str
    size: float


@dataclass(frozen=True, slots=True)
class BackupComplete(BackupBaseFields, ResponseData):
    """BackupComplete model."""

    supervisor_version: str
    homeassistant: str | None
    addons: list[BackupAddon]
    repositories: list[str]
    folders: list[Folder]
    homeassistant_exclude_database: bool | None
    extra: dict | None


@dataclass(frozen=True, slots=True)
class BackupList(ResponseData):
    """BackupList model."""

    backups: list[Backup]


@dataclass(frozen=True, slots=True)
class BackupsInfo(BackupList):
    """BackupsInfo model."""

    days_until_stale: int


@dataclass(frozen=True, slots=True)
class BackupsOptions(Request):
    """BackupsOptions model."""

    days_until_stale: int


@dataclass(frozen=True, slots=True)
class FreezeOptions(Request):
    """FreezeOptions model."""

    timeout: int


@dataclass(frozen=True)
class PartialBackupRestoreOptions(ABC):  # noqa: B024
    """PartialBackupRestoreOptions ABC type."""

    addons: set[str] | None = None
    folders: set[Folder] | None = None
    homeassistant: bool | None = None

    def __post_init__(self) -> None:
        """Validate at least one thing to backup/restore is included."""
        if not any((self.addons, self.folders, self.homeassistant)):
            raise ValueError(
                "At least one of addons, folders, or homeassistant must have a value"
            )


@dataclass(frozen=True, slots=True)
class FullBackupOptions(Request):
    """FullBackupOptions model."""

    name: str | None = None
    password: str | None = None
    compressed: bool | None = None
    location: str | list[str | None] | None = DEFAULT  # type: ignore[assignment]
    homeassistant_exclude_database: bool | None = None
    background: bool | None = None
    extra: dict | None = None


@dataclass(frozen=True, slots=True)
class PartialBackupOptions(FullBackupOptions, PartialBackupRestoreOptions):
    """PartialBackupOptions model."""

    addons: AddonSet | set[str] | None = None


@dataclass(frozen=True, slots=True)
class BackupJob(ResponseData):
    """BackupJob model."""

    job_id: str


@dataclass(frozen=True, slots=True)
class NewBackup(BackupJob):
    """NewBackup model."""

    slug: str | None = None


@dataclass(frozen=True, slots=True)
class FullRestoreOptions(Request):
    """FullRestoreOptions model."""

    password: str | None = None
    background: bool | None = None
    location: str | None = DEFAULT  # type: ignore[assignment]


@dataclass(frozen=True, slots=True)
class PartialRestoreOptions(FullRestoreOptions, PartialBackupRestoreOptions):
    """PartialRestoreOptions model."""


@dataclass(frozen=True, slots=True)
class UploadBackupOptions(Request):
    """UploadBackupOptions model."""

    location: set[str | None] = None


@dataclass(frozen=True, slots=True)
class UploadedBackup(ResponseData):
    """UploadedBackup model."""

    slug: str


@dataclass(frozen=True, slots=True)
class RemoveBackupOptions(Request):
    """RemoveBackupOptions model."""

    location: set[str | None] = None


@dataclass(frozen=True, slots=True)
class DownloadBackupOptions(Request):
    """DownloadBackupOptions model."""

    location: str | None = DEFAULT  # type: ignore[assignment]



"""Utilities for interacting with aiohttp."""

from typing import Self

from aiohttp import StreamReader


class ChunkAsyncStreamIterator:
    """Async iterator for chunked streams.

    Based on aiohttp.streams.ChunkTupleAsyncStreamIterator, but yields
    bytes instead of tuple[bytes, bool].
    Borrowed from home-assistant/core.
    """

    __slots__ = ("_stream",)

    def __init__(self, stream: StreamReader) -> None:
        """Initialize."""
        self._stream = stream

    def __aiter__(self) -> Self:
        """Iterate."""
        return self

    async def __anext__(self) -> bytes:
        """Yield next chunk."""
        rv = await self._stream.readchunk()
        if rv == (b"", False):
            raise StopAsyncIteration
        return rv[0]



"""Test network supervisor client."""

from ipaddress import IPv4Address, IPv4Interface

from aioresponses import aioresponses
import pytest
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import (
    InterfaceMethod,
    IPv4Config,
    NetworkInterfaceConfig,
    VlanConfig,
)

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_network_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test network info API."""
    responses.get(
        f"{SUPERVISOR_URL}/network/info",
        status=200,
        body=load_fixture("network_info.json"),
    )
    result = await supervisor_client.network.info()
    assert result.interfaces[0].interface == "end0"
    assert result.interfaces[0].type == "ethernet"
    assert result.interfaces[0].enabled is True
    assert result.interfaces[0].mac == "00:11:22:33:44:55"
    assert result.interfaces[0].ipv4.method == "static"
    assert result.interfaces[0].ipv4.address[0].with_prefixlen == "192.168.1.2/24"
    assert result.interfaces[0].ipv4.nameservers[0].compressed == "192.168.1.1"
    assert result.interfaces[0].ipv4.gateway.compressed == "192.168.1.1"
    assert result.interfaces[0].ipv4.ready is True
    assert result.interfaces[0].ipv6.method == "disabled"
    assert (
        result.interfaces[0].ipv6.address[0].with_prefixlen
        == "fe80::819d:c479:d712:7a77/64"
    )
    assert result.interfaces[0].ipv6.gateway is None
    assert result.interfaces[0].wifi is None
    assert result.interfaces[0].vlan is None

    assert result.docker.interface == "hassio"
    assert result.docker.address.compressed == "172.30.32.0/23"
    assert result.docker.gateway.compressed == "172.30.32.1"
    assert result.docker.dns.compressed == "172.30.32.3"
    assert result.host_internet is True
    assert result.supervisor_internet is True


async def test_network_reload(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test network reload API."""
    responses.post(f"{SUPERVISOR_URL}/network/reload", status=200)
    assert await supervisor_client.network.reload() is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/network/reload"))
    }


async def test_network_interface_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test network interface info API."""
    responses.get(
        f"{SUPERVISOR_URL}/network/interface/end0/info",
        status=200,
        body=load_fixture("network_interface_info.json"),
    )
    result = await supervisor_client.network.interface_info("end0")
    assert result.interface == "end0"
    assert result.type == "ethernet"
    assert result.enabled is True
    assert result.mac == "00:11:22:33:44:55"
    assert result.ipv4.method == "static"
    assert result.ipv4.address[0].with_prefixlen == "192.168.1.2/24"
    assert result.ipv4.nameservers[0].compressed == "192.168.1.1"
    assert result.ipv4.gateway.compressed == "192.168.1.1"
    assert result.ipv4.ready is True
    assert result.ipv6.method == "disabled"
    assert result.ipv6.address[0].with_prefixlen == "fe80::819d:c479:d712:7a77/64"
    assert result.ipv6.gateway is None
    assert result.wifi is None
    assert result.vlan is None


async def test_network_update_interface(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test network interface update API."""
    responses.post(f"{SUPERVISOR_URL}/network/interface/end0/update", status=200)
    config = NetworkInterfaceConfig(
        ipv4=IPv4Config(
            method=InterfaceMethod.STATIC,
            address=[IPv4Interface("192.168.1.2/24")],
            gateway=IPv4Address("192.168.1.1"),
            nameservers=[IPv4Address("192.168.1.1")],
        )
    )
    assert (
        await supervisor_client.network.update_interface("end0", config=config) is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/network/interface/end0/update"))
    }


async def test_network_access_points(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test network access points API."""
    responses.get(
        f"{SUPERVISOR_URL}/network/interface/end0/accesspoints",
        status=200,
        body=load_fixture("network_access_points.json"),
    )
    result = await supervisor_client.network.access_points("end0")
    assert result[0].mode == "infrastructure"
    assert result[0].ssid == "UPC4814466"
    assert result[0].frequency == 2462
    assert result[0].signal == 47
    assert result[0].mac == "AA:BB:CC:DD:EE:FF"
    assert result[1].ssid == "VQ@35(55720"


@pytest.mark.parametrize(
    "config",
    [None, NetworkInterfaceConfig(ipv4=IPv4Config(method=InterfaceMethod.AUTO))],
)
async def test_network_save_vlan(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    config: NetworkInterfaceConfig | None,
) -> None:
    """Test network save vlan API."""
    responses.post(f"{SUPERVISOR_URL}/network/interface/end0/vlan/1", status=200)
    assert await supervisor_client.network.save_vlan("end0", 1, config=config) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/network/interface/end0/vlan/1"))
    }


async def test_network_configs_cannot_be_empty() -> None:
    """Test network config instances require at least one field specified."""
    # Network interface config for update calls
    with pytest.raises(ValueError, match="At least one field must have a value"):
        NetworkInterfaceConfig()

    # Vlan config for save vlan calls
    with pytest.raises(ValueError, match="At least one field must have a value"):
        VlanConfig()
